function [data, dict_size] = generate_bar_data(dict_size, num_docs, num_topics)
% GENERATE_BAR_DATA - Generates synthetic data from the 'bars' dataset (see
% below for ref).
%    generate_bar_data(dict_size, num_docs, num_topics)
%
%    dict_size should be a squared number (e.g., 16, 25, 36, etc.).
%    num_docs is the number of documents to generate.
%    num_topics is the number of topics to generate.
%      Should be <= 2*sqrt(dict_size)
%
%   T. L. Griffiths and M. Steyvers. Finding scientific topics. Proceedings
%   of the National Academy of Sciences, 101:5228–5235, April 2004.
%
%   Copyright(c) 2014. Jason Chang, CSAIL, MIT. 

if (~exist('dict_size', 'var') || isempty(dict_size))
    dict_size = 25;
end
if (~exist('num_docs', 'var') || isempty(num_docs))
    num_docs = 100;
end
if (~exist('num_topics', 'var') || isempty(num_topics))
    num_topics = 10;
end

topicDistributions = zeros(dict_size,num_topics);
topicDistributionsCDF = zeros(dict_size,num_topics);
for k=1:num_topics
    dist = zeros(5);
    if (mod(k,2)==0)
        dist(floor((k-1)/2)+1,:) = 1;
    else
        dist(:,floor((k-1)/2)+1) = 1;
    end
    dist = dist / sum(dist(:));
    topicDistributions(:,k) = dist(:);
    topicDistributionsCDF(:,k) = cumsum(topicDistributions(:,k));
end

data = struct('words',[]);
data(num_docs).words = [];

for d=1:num_docs
    Nd = randi(100) + 50;
    
    words = zeros(1,Nd,'uint32');
    topicProbs = dirrnd(0.1*ones(1,num_topics));
    topicCDFs = cumsum(topicProbs);
    topicRands = rand(Nd,1);
    topicInds = num_topics+1-sum(bsxfun(@le, topicRands, topicCDFs),2);
    
    for i=1:Nd
        words(i) = find(rand <= topicDistributionsCDF(:,topicInds(i)),1,'first')-1;
    end
    data(d).words = words;
end