

#include <vector>
#include "dir_sampled.h"
#include "dir_sampled_full.h"
using std::vector;

#ifdef USEFULL
   typedef dir_sampled_full dirichlet;
#else
   typedef dir_sampled dirichlet;
#endif

typedef float Real;

struct dishComps
{
	int dishInd;
   Real beta;
   dirichlet param;
   int numTables;
   dishComps() {}
   dishComps(int _dishInd, Real _beta, dirichlet& _param, int _numTables) :
      dishInd(_dishInd), beta(_beta), param(_param), numTables(_numTables) {}

#ifdef USELOGLIKELIHOOD
   Real predictive_loglikelihood(int word);
#else
   Real predictive_likelihood(int word);
#endif

#ifdef USEFULL
   void set_stats(int N, const int* indices_counts);
#else
   void set_stats(int N, const unordered_map<int,int>& indices_counts);
#endif
   
   void setBeta(Real _beta);
   Real getBeta();
   int getN();
   Real data_loglikelihood_marginalized();
   void empty_params();
   Real normalizeBeta(Real total);
   void temp_merge_with(dishComps& other);
   void sample();
};

#ifdef USELOGLIKELIHOOD
inline Real dishComps::predictive_loglikelihood(int word)
{
	return param.predictive_loglikelihood(word);
}
#else
inline Real dishComps::predictive_likelihood(int word)
{
	return param.predictive_likelihood(word);
}
#endif

#ifdef USEFULL
inline void dishComps::set_stats(int N, const int* indices_counts)
{
	param.set_stats(N, indices_counts);
}
#else
inline void dishComps::set_stats(int N, const unordered_map<int,int>& indices_counts)
{
	param.set_stats(N, indices_counts);
}
#endif

inline void dishComps::setBeta(Real _beta)			{ beta=_beta; }
inline Real dishComps::getBeta()							{ return beta; }
inline int dishComps::getN()									{ return param.getN(); }
inline Real dishComps::data_loglikelihood_marginalized()
{
	return param.data_loglikelihood_marginalized();
}
inline void dishComps::empty_params()
{
	param.empty();
}
inline Real dishComps::normalizeBeta(Real total)
{
	beta /= total;
	return beta;
}
inline void dishComps::temp_merge_with(dishComps& other)
{
	beta += other.beta;
	numTables += other.numTables;
	param.temp_merge_with(&(other.param));
}
inline void dishComps::sample()
{
	param.sample();
}


struct dish
{
	int dishInd;
   dishComps main;
   dishComps left;
   dishComps right;
   Real logpz;
   
   Real subloglikelihoodOld;
   Real subloglikelihoodDelta;
   bool splittable;

   Real loglikelihoodOld;
   Real loglikelihoodDelta;
   bool mergeable;

   dish() {}
   dish(int _dishInd, Real beta, double* betaSubs, dirichlet& hyper,
   	Real sll, Real slld, bool _splittable, Real ll, Real lld, bool _mergeable);
   dish(int _dishInd, Real beta, float* betaSubs, dirichlet& hyper,
   	Real sll, Real slld, bool _splittable, Real ll, Real lld, bool _mergeable);
   void add_data_inc_all(int c, int w);

   int getN();
   int getNL();
   int getNR();
   Real getBeta();
   Real getBetaL();
   Real getBetaR();
	int getNumTables();
   int getNumTablesL();
   int getNumTablesR();

   //int getNSub(int s);
   //Real getBetaSub(int s);
   //Real getNumTablesSub(int s);

#ifdef USELOGLIKELIHOOD
   Real predictive_loglikelihood(int word);
   Real predictive_loglikelihoodL(int word);
   Real predictive_loglikelihoodR(int word);
#else
   Real predictive_likelihood(int word);
   Real predictive_likelihoodL(int word);
   Real predictive_likelihoodR(int word);
#endif

   //Real predictive_loglikelihoodSub(int s, int word);
   Real predictive_loglikelihood_marginalized(int word);

   Real data_loglikelihood_marginalized();
   Real data_loglikelihood_marginalizedL();
   Real data_loglikelihood_marginalizedR();

   //Real data_loglikelihood_marginalizedSub(int s);

   Real data_loglikelihood_marginalized_testmerge(dish& other);

   void sample();
   void sample_subdishes();
   void setBeta(Real beta);
   void setBeta(Real beta, Real betaL, Real betaR);
   Real sampleBeta(gsl_rng* r, Real gamma);
   Real normalizeBeta(Real total);

   void empty_params();

   void set_unsplittable();
   void set_unmergeable();

#ifdef USEFULL
   void set_stats(int* N, int* indices_counts);
#else
   void set_stats(int* N, unordered_map<int,int>* indices_counts);
#endif
   void set_numTables(int numTables, int* numTablesSubs);

   void clear_numTables();
   void add_numTables(int numTables);

	void merge_with(dish& other);
   void split_fromSub(int s, dish& other);
	void split_fix();
};


inline dish::dish(int _dishInd, Real beta, float* betaSubs, dirichlet& hyper,
	Real sll, Real slld, bool _splittable, Real ll, Real lld, bool _mergeable) : 
   dishInd(_dishInd), main(_dishInd,beta,hyper,0), left(_dishInd,betaSubs[0],hyper,0), right(_dishInd,betaSubs[1],hyper,0),
   splittable(_splittable), subloglikelihoodOld(sll), subloglikelihoodDelta(slld),
   mergeable(_mergeable), loglikelihoodOld(ll), loglikelihoodDelta(lld)
{
}

inline dish::dish(int _dishInd, Real beta, double* betaSubs, dirichlet& hyper,
	Real sll, Real slld, bool _splittable, Real ll, Real lld, bool _mergeable) : 
   dishInd(_dishInd), main(_dishInd,beta,hyper,0), left(_dishInd,betaSubs[0],hyper,0), right(_dishInd,betaSubs[1],hyper,0),
   splittable(_splittable), subloglikelihoodOld(sll), subloglikelihoodDelta(slld),
   mergeable(_mergeable), loglikelihoodOld(ll), loglikelihoodDelta(lld)
{
}

inline void dish::add_data_inc_all(int c, int w)
{
	main.param.add_data_inc(w);
	if ((c%2)==0)
		left.param.add_data_inc(w);
	else
		right.param.add_data_inc(w);
}

inline int dish::getN()								{ return main.param.getN(); }
inline int dish::getNL()							{ return left.param.getN(); }
inline int dish::getNR()							{ return right.param.getN(); }

inline Real dish::getBeta()						{ return main.beta; }
inline Real dish::getBetaL()					{ return left.beta; }
inline Real dish::getBetaR()					{ return right.beta; }

inline int dish::getNumTables()				{ return main.numTables; }
inline int dish::getNumTablesL()				{ return left.numTables; }
inline int dish::getNumTablesR()				{ return right.numTables; }

//inline int dish::getNSub(int s) { return (s==0) ? left.param.getN() : right.param.getN(); }
//inline Real dish::getBetaSub(int s) { return (s==0) ? left.beta : right.beta; }
//inline Real dish::getNumTablesSub(int s) { return (s==0) ? left.numTables : right.numTables;}

#ifdef USELOGLIKELIHOOD
inline Real dish::predictive_loglikelihood(int word)
{
	return main.param.predictive_loglikelihood(word);
}
inline Real dish::predictive_loglikelihoodL(int word)
{
	return left.param.predictive_loglikelihood(word);
}
inline Real dish::predictive_loglikelihoodR(int word)
{
	return right.param.predictive_loglikelihood(word);
}
#else
inline Real dish::predictive_likelihood(int word)
{
	return main.param.predictive_likelihood(word);
}
inline Real dish::predictive_likelihoodL(int word)
{
	return left.param.predictive_likelihood(word);
}
inline Real dish::predictive_likelihoodR(int word)
{
	return right.param.predictive_likelihood(word);
}
#endif

/*inline Real dish::predictive_loglikelihoodSub(int s, int word)
{
	if (s==0)
		return left.param.predictive_loglikelihood(word);
	else
		return right.param.predictive_loglikelihood(word);
}*/

inline Real dish::predictive_loglikelihood_marginalized(int word)
{
	return main.param.predictive_loglikelihood_marginalized(word);
}

inline Real dish::data_loglikelihood_marginalized()
{
	return main.param.data_loglikelihood_marginalized();
}
inline Real dish::data_loglikelihood_marginalizedL()
{
	return left.param.data_loglikelihood_marginalized();
}
inline Real dish::data_loglikelihood_marginalizedR()
{
	return right.param.data_loglikelihood_marginalized();
}
/*inline Real dish::data_loglikelihood_marginalizedSub(int s)
{
	if (s==0)
		return left.param.data_loglikelihood_marginalized();
	else
		return right.param.data_loglikelihood_marginalized();
}*/

inline Real dish::data_loglikelihood_marginalized_testmerge(dish& other)
{
	return main.param.data_loglikelihood_marginalized_testmerge(&(other.main.param));
}


inline void dish::sample()
{
	main.param.sample();
	sample_subdishes();
}
inline void dish::sample_subdishes()
{
	left.param.sample();
	right.param.sample();
}

inline void dish::setBeta(Real beta)
{
	main.beta = beta;
}
inline void dish::setBeta(Real beta, Real betaL, Real betaR)
{
	main.beta = beta;
	left.beta = betaL;
	right.beta = betaR;
}

inline Real dish::sampleBeta(gsl_rng* r, Real gamma)
{
	main.beta = gsl_ran_gamma(r, main.numTables, 1);

	left.beta = gsl_ran_gamma(r, gamma + left.numTables, 1);
	right.beta = gsl_ran_gamma(r, gamma + right.numTables, 1);
	Real total = left.beta + right.beta;
	left.beta /= total;
	right.beta /= total;

	return main.beta;
}
inline Real dish::normalizeBeta(Real total)
{
	main.beta /= total;
	return main.beta;
}

inline void dish::empty_params()
{
	main.param.empty();
	left.param.empty();
	right.param.empty();
}

inline void dish::set_unsplittable()
{
	subloglikelihoodOld = mxGetInf();
   subloglikelihoodDelta = -mxGetInf();
   splittable = false;
}
inline void dish::set_unmergeable()
{
	loglikelihoodOld = mxGetInf();
   loglikelihoodDelta = -mxGetInf();
   mergeable = false;
}


#ifdef USEFULL
inline void dish::set_stats(int* N, int* indices_counts)
{
	int totalN = 0;
	main.param.empty();
	left.param.set_stats(N[0], indices_counts);
	right.param.set_stats(N[1], indices_counts+main.param.getD());

	main.param.add_data(indices_counts);
	main.param.add_data(indices_counts+main.param.getD());
}
#else
inline void dish::set_stats(int* N, unordered_map<int,int>* indices_counts)
{
	int totalN = 0;
	main.param.empty();
	left.param.set_stats(N[0], indices_counts[0]);
	right.param.set_stats(N[1], indices_counts[1]);

	main.param.add_data(indices_counts[0]);
	main.param.add_data(indices_counts[1]);
}
#endif

inline void dish::set_numTables(int numTables, int* numTablesSubs)
{
	main.numTables = numTables;
	left.numTables = numTablesSubs[0];
	right.numTables = numTablesSubs[1];
}

inline void dish::clear_numTables()
{
	main.numTables = 0;
}
inline void dish::add_numTables(int numTables)
{
	main.numTables += numTables;
}


inline void dish::split_fromSub(int s, dish& other)
{
	// transfer the betas
	if (s==0)
	{
		main.beta = other.main.beta * other.left.beta;
		main.numTables = other.left.numTables;
		main.param = other.left.param;
	}
	else
	{
		main.beta = other.main.beta * other.right.beta;
		main.numTables = other.right.numTables;
		main.param = other.right.param;
	}

	left.beta = 0.5;
	right.beta = 0.5;
}

inline void dish::merge_with(dish& other)
{
	main.beta += other.main.beta;
	main.numTables += other.main.numTables;
}