#ifndef _HDP_SUBCLUSTERS_H_INCLUDED_
#define _HDP_SUBCLUSTERS_H_INCLUDED_

#include "dir_sampled.h"
#include "dir_sampled_full.h"
#include <boost/unordered_map.hpp>
#include <vector>
#include "corpus.h"
#include "sample_categorical.h"
#include "reduction_hash2.h"
#include "reduction_array.h"
#include "reduction_array2.h"
#include "dish.h"
#include "restaurantDish.h"

#include "gsl/gsl_permutation.h"

using boost::unordered_map;
using std::pair;
using std::vector;


#ifdef USEFULL
   typedef dir_sampled_full dirichlet;
#else
   typedef dir_sampled dirichlet;
#endif

typedef float Real;


class hdp_subclusters
{
private:
   int NThreads;

	int D; // D documents
	int K; // number of unique topics
	corpus documents;

   bool always_splittable;

	float gamma, loggamma;
	Real alpha, logalpha;
	Real diralpha;

   int logstirlingW;
   arr(double) logstirling;
   int bestNumTablesN;
   arr(int) bestNumTables;

   Real emptyBeta;
	vector< vector<int> > customers; // direct topic assignment
   vector< vector<int> > randCustomers;
   vector< unordered_map<int,restaurantDish> > restaurants; // keeps track of relevant counts for each dish
	unordered_map<int,dish> dishes; // topic with counts on how many tables have the particular dish
	dirichlet hyper;

   vector<dish*> k2zDishPtrs;
   vector<int> z2k;

   vector<Real> logpzMtx;

   vector<int> docInds;
   vector<int> wordInds;



   vector< vector<int> > tcustomers;


	// random number generation
   arr(gsl_rng*) rArr;
   arr(arr(Real)) probsArr;
   arr(arr(int)) probsArrIndices;

public:
   // --------------------------------------------------------------------------
   // -- hdp_subclusters
   // --   copy constructor;
   // --------------------------------------------------------------------------
   hdp_subclusters(const hdp_subclusters& that);
   // --------------------------------------------------------------------------
   // -- operator=
   // --   assignment operator
   // --------------------------------------------------------------------------
   hdp_subclusters& operator=(const hdp_subclusters& that);
   // --------------------------------------------------------------------------
   // -- copy
   // --   returns a copy of this
   // --------------------------------------------------------------------------
   void copy(const hdp_subclusters& that);
   // --------------------------------------------------------------------------
   // -- hdp_subclusters
   // --   constructor; intializes to all the values given
   // --------------------------------------------------------------------------
   hdp_subclusters(const mxArray* params, const mxArray* _data);

   // --------------------------------------------------------------------------
   // -- ~hdp_subclusters
   // --   destructor
   // --------------------------------------------------------------------------
   virtual ~hdp_subclusters();

   void cleanup();

   // --------------------------------------------------------------------------
   // -- initialize
   // --   populates the initial statistics
   // --------------------------------------------------------------------------
   void initialize(const mxArray* inputs);

   // --------------------------------------------------------------------------
   // -- write_output
   // --   creates and writes the output cluster structure
   // --------------------------------------------------------------------------
   void write_output(mxArray* &plhs);

   Real joint_loglikelihood();
   Real HOW_loglikelihood();

   void populate_k2z();
   int getK();
   void getnhist(vector<int>& mvals);

   void deleteDish(int dishInd);

   void sample_thetas();
   void sample_pis();
   void sample_params();
   void sample_dishes2();
   void sample_dishes3();
   void sample_counts();
   void sample_betas();

   void sample_hyperparameters();

   void propose_splits1();


   void merge2_find_bestNumTables(vector<int>& merge_proposals, int Ks,
      vector<int>& moldL, vector<int>& moldR);
   void merge2_mergeCustomersLoglikelihood(vector<int>& merge_proposals,
      int Ks, vector<Real>& loglikelihoods);
   void propose_merges2();

   void split2_find_bestNumTables(vector<int>& mnewL,
      vector<int>& mnewR);
   void split2_sample_betas_bestNumTables(vector<int> &mnewL, vector<int>& mnewR,
      vector<Real> &betaL, vector<Real> &betaR);
   void split2_splitCustomersLoglikelihood(const vector<Real> &newBetaL, const vector<Real> &newBetaR,
      vector<Real>& loglikelihoods);
   void propose_splits2();


   Real splitmerge_sample_ztilde(
      vector< unordered_map<int,restaurantDishComps> > &trestaurants,
      vector<dishComps*> &k2ztDishPtrs);
   void splitmerge_find_bestNumTables(vector< unordered_map<int,restaurantDishComps> > &trestaurants,
      vector<dishComps*> &k2ztDishPtrs, vector<int>& mnew, vector<int>& mold);
   void splitmerge_sample_betas_bestNumTables(vector<dishComps*> &k2ztDishPtrs,
      vector<int> &mnew, vector<int> &mold, int& mnewTotal, int& moldTotal);
   void splitmerge_sample_temporary_params(vector< unordered_map<int,restaurantDishComps> > &trestaurants,
      vector<dishComps*> &k2ztDishPtrs);


   void split_create_temporary_vars(int splitk,
      vector< unordered_map<int,restaurantDishComps> > &trestaurants,
      unordered_map<int,dishComps> &tdishes, int &oldDishInd, int &newDishInd);
   Real split_loglikelihood_zmerge(int splitk, int oldDishInd, int newDishInd,
      vector< unordered_map<int,restaurantDishComps> > &trestaurants,
      unordered_map<int,dishComps> &tdishes, vector<dishComps*> &k2ztDishPtrs);
   void propose_splits3();

   void merge_create_temporary_vars(int k1, int k2,
      vector< unordered_map<int,restaurantDishComps> > &trestaurants,
      unordered_map<int,dishComps> &tdishes, int &oldDishInd, int &newDishInd);
   Real merge_loglikelihood_zsplit(int k1, int k2,
      vector< unordered_map<int,restaurantDishComps> > &trestaurants,
      unordered_map<int,dishComps> &tdishes, vector<dishComps*> &k2ztDishPtrs);
   void propose_merges3();

   void merger_mergeCustomersLoglikelihood(vector<int>& merge_proposals,
      int Ks, vector<Real>& loglikelihoods);
   void propose_mergesr();
   void construct_splitsr(vector<Real> &splitLoglikelihoods,
      vector<int> &mnewL, vector<int> &mnewR);
   void splitr_sample_betas_bestNumTables(vector<int> &mnewL, vector<int>& mnewR,
      vector<Real> &betaL, vector<Real> &betaR);
   void splitr_splitCustomersLoglikelihood(const vector<Real> &newBetaL, const vector<Real> &newBetaR,
      vector<Real>& loglikelihoods);
   void propose_splitsr();
   

   void propose_random_split_assignments();
   void propose_random_splits();
   void propose_random_merges();

   void check();
};

#endif