// =============================================================================
// == calc_sdfIMPORT.cpp
// == --------------------------------------------------------------------------
// == The MEX interface file to calculate a signed distance function
// == --------------------------------------------------------------------------
// == Copyright 2011. MIT. All Rights Reserved.
// == Written by Jason Chang 06-13-2011
// =============================================================================

#include "mex.h"
#include <iostream>
#include <math.h>
#include <algorithm>
#include <time.h>
#include "debugMEX.h"
#include "helperMEX.h"
#include "hdp_subclusters.h"
#include "dir_sampled.h"
#include "stopwatch.h"

#define NUMARGS 3
#define NUMOUT 4


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
   // Check for proper number of arguments
   if (nrhs != NUMARGS) {
         mexErrMsgTxt("Incorrect number of input arguments required.");
   } else if (nlhs > NUMOUT) {
         mexErrMsgTxt("Too many output arguments expected.");
   }

   checkInput(prhs[0], STRUCT); //data
   checkInput(prhs[1], STRUCT); // params
   checkInput(prhs[2], STRUCT); // model

   const mxArray* data = prhs[0];
   const mxArray* params = prhs[1];

   int its_crf = getInput<double>(getField(params,0,"its_crf"));
   int NThreads = getInput<double>(getField(params,0,"NThreads"));

   omp_set_num_threads(NThreads);

   const mxArray* model = prhs[2];


   plhs[0] = mxCreateNumericMatrix(1,1,mxDOUBLE_CLASS,mxREAL);
   plhs[1] = mxCreateNumericMatrix(1,1,mxDOUBLE_CLASS,mxREAL);
   plhs[2] = mxCreateNumericMatrix(1,1,mxDOUBLE_CLASS,mxREAL);
   plhs[3] = mxCreateCellMatrix(1,1);
   arr(double) EArr = getArrayInput<double>(plhs[0]);
   arr(double) HOWArr = getArrayInput<double>(plhs[1]);
   arr(double) KnnzArr = getArrayInput<double>(plhs[2]);
   mxArray* nhistArr = plhs[3];


   hdp_subclusters hdp(params, data);
   hdp.initialize(model);
   hdp.sample_params();
   EArr[0] = hdp.joint_loglikelihood();
   HOWArr[0] = hdp.HOW_loglikelihood();
   KnnzArr[0] = hdp.getK();

   vector<int> nhistvals;
   hdp.getnhist(nhistvals);
   mxSetCell(nhistArr, 0, mxCreateArrayINT32<int>(nhistvals.size(),1, nhistvals.data()));
}
