#ifndef _STOPWATCH_H_INCLUDED_
#define _STOPWATCH_H_INCLUDED_


#ifdef OSMAC
   #include <mach/mach_time.h>
   #define CLOCK_REALTIME 0
   #define CLOCK_MONOTONIC 0
   int clock_gettime(int clk_id, struct timespec *t){
       mach_timebase_info_data_t timebase;
       mach_timebase_info(&timebase);
       uint64_t time;
       time = mach_absolute_time();
       double nseconds = ((double)time * (double)timebase.numer)/((double)timebase.denom);
       double seconds = ((double)time * (double)timebase.numer)/((double)timebase.denom * 1e9);
       t->tv_sec = seconds;
       t->tv_nsec = nseconds;
       return 0;
   }
#else
   #ifdef OSWINDOWS

      #define CLOCK_REALTIME 0
      #define CLOCK_MONOTONIC 0
      struct timespec {
          time_t   tv_sec;        /* seconds */
          long     tv_nsec;       /* nanoseconds */
      };

      //#define _WINSOCKAPI_
      //#include <Windows.h>
      int clock_gettime(int clk_id, struct timespec *t){
         //int curtime = GetTickCount();
         int curtime = 0;
         t->tv_sec = curtime/1000;
         t->tv_nsec = curtime%1000 * 1000000;
         return 0;
      }
   #else
      #include <time.h>
   #endif
#endif

class stopwatch
{
private:
   timespec start;
   timespec stop;
public:
   void tic();
   double toc();
};

timespec diff(timespec start, timespec end);

inline void stopwatch::tic()
{
   clock_gettime(CLOCK_MONOTONIC, &start);
}
inline double stopwatch::toc()
{
   clock_gettime(CLOCK_MONOTONIC, &stop);
   return (diff(start,stop).tv_sec) + (double)(diff(start,stop).tv_nsec)*1e-9;
}

// calculate the difference in times
timespec diff(timespec start, timespec end)
{
   timespec ts;
   ts.tv_sec = end.tv_sec - start.tv_sec;
   ts.tv_nsec = end.tv_nsec - start.tv_nsec;
   if (ts.tv_nsec < 0) {
      ts.tv_sec--;
      ts.tv_nsec += 1000000000;
   }
   return ts;
}

#endif
